@@ -39,7 +39,7 @@ The tarball_element default configuration:
 
 import tarfile
 
-from buildstream import Element, Scope, ElementError
+from buildstream import Element, ElementError
 
 
 class TarElement(Element):
@@ -85,19 +85,15 @@ class TarElement(Element):
         pass
 
     def stage(self, sandbox):
-        pass
+        # Stage deps in the sandbox at "/input"
+        with self.timed_activity("Staging dependencies", silent_nested=True):
+            self.stage_dependency_artifacts(sandbox, path="/input")
 
     def assemble(self, sandbox):
         basedir = sandbox.get_virtual_directory()
         inputdir = basedir.descend("input", create=True)
         outputdir = basedir.descend("output", create=True)
 
-        # Stage deps in the sandbox root
-        with self.timed_activity("Staging dependencies", silent_nested=True):
-            self.stage_dependency_artifacts(
-                sandbox, Scope.BUILD, path="/input"
-            )
-
         with self.timed_activity("Creating tarball", silent_nested=True):
 
             # Create an uncompressed tar archive
