@@ -61,7 +61,7 @@ import tarfile
 import urllib.error
 import urllib.request
 
-import pytoml
+import toml
 from buildstream import Source, SourceFetcher, SourceError
 from buildstream import utils
 
@@ -148,7 +148,7 @@ class Crate(SourceFetcher):
                 checksum_file = os.path.join(
                     package_dir, ".cargo-checksum.json"
                 )
-                with open(checksum_file, "w") as f:
+                with open(checksum_file, "w", encoding="utf-8") as f:
                     checksum_data = {"package": self.sha, "files": {}}
                     json.dump(checksum_data, f)
 
@@ -287,7 +287,7 @@ class Crate(SourceFetcher):
             self._get_mirror_dir(), "{}.etag".format(sha)
         )
         if os.path.exists(etagfilename):
-            with open(etagfilename, "r") as etagfile:
+            with open(etagfilename, "r", encoding="utf-8") as etagfile:
                 return etagfile.read()
 
         return None
@@ -389,10 +389,10 @@ class CargoSource(Source):
         lockfile = os.path.join(previous_sources_dir, self.cargo_lock)
 
         try:
-            with open(lockfile, "r") as f:
+            with open(lockfile, "r", encoding="utf-8") as f:
                 try:
-                    lock = pytoml.load(f)
-                except pytoml.core.TomlError as e:
+                    lock = toml.load(f)
+                except toml.TomlDecodeError as e:
                     raise SourceError(
                         "Malformed Cargo.lock file at: {}".format(
                             self.cargo_lock
@@ -447,7 +447,7 @@ class CargoSource(Source):
         conf_dir = os.path.join(directory, ".cargo")
         conf_file = os.path.join(conf_dir, "config")
         os.makedirs(conf_dir, exist_ok=True)
-        with open(conf_file, "w") as f:
+        with open(conf_file, "w", encoding="utf-8") as f:
             f.write(vendor_config)
 
     def get_source_fetchers(self):
